package com.link.core.basic.interceptor;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.link.base.enterprise.model.CoreEnterprise;
import com.link.base.orgtile.CoreOrganizationTile;
import com.link.base.user.model.CoreUser;
import com.link.core.util.AesUtil;
import com.link.core.util.RedisUtil;
import com.link.core.util.StringUtils;
import com.link.core.util.redisclient.LinkRedisClient;
import java.util.Enumeration;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.ThreadContext;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

public class SecurityInterceptor extends HandlerInterceptorAdapter implements HandlerInterceptor {
   private static final Logger logger = LogManager.getLogger(SecurityInterceptor.class);
   public static final String NO_INTERCEPT_FLAG_NAME = "NoIntercept";
   public static final String NO_INTERCEPT_FLAG_VALUE = "NoIntercept";
   private static final String MP = "MP";
   private static final String IGNORE = "Ignore";
   private static final String CHECKDATEIGNORE = "CheckDataIgnore";
   public static final int TOKEN_VALID_TIME = 86400;
   public static final String LOGIN_INFO_ENTERPRISE_FIELD = "enterprise";
   public static final String ACCESS_TOKEN_FIELD = "access_token";
   public static final String TOKEN_SCOPE = "token_scope";
   public static final String TOKEN_PREFIX = "auth_user_";
   public static final String INNER_TOKEN = "INNERTOKEN";
   public static final String USER_KEY = "user";
   public static final String LOGIN_INFO_PREFIX = "login_info_";
   public static final String CUSTOM_TOKEN_KEY = "customToken";
   public static final String ENTERPRISE_KEY = "enterprise";
   public static final String ORG_TILE_KEY = "orgTile";
   public static final String INTERNAL_TOKEN = "internal_token";
   public static final String AUTHORIZATION = "Authorization";
   public static final ThreadLocal<CoreUser> tempUser = new ThreadLocal();
   public static final ThreadLocal<CoreEnterprise> tempEnterpirse = new ThreadLocal();
   public static final ThreadLocal<Long> reqBegin = new ThreadLocal();
   public static final ThreadLocal<String> token = new ThreadLocal();
   public static final ThreadLocal<String> internal_token = new ThreadLocal();
   public static final ThreadLocal<String> linkRequestId = new ThreadLocal();

   public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
      Enumeration<String> headerNames = request.getHeaderNames();
      reqBegin.set(System.currentTimeMillis());
      String allowVisitFlag = request.getHeader("NoIntercept");
      String curRequestId = request.getHeader("LinkRequestId");
      ThreadContext.put("LinkRequestId", curRequestId);
      linkRequestId.set(curRequestId);
      if ("NoIntercept".equalsIgnoreCase(allowVisitFlag)) {
         return true;
      } else {
         String innerToken = request.getHeader("INNERTOKEN");
         String customToken = request.getHeader("customToken");
         String tokenKey = request.getHeader("Authorization");
         String token = this.splitOrginToken(tokenKey);
         SecurityInterceptor.token.set(token);
         if (StringUtils.isNotBlank(customToken)) {
            SecurityInterceptor.token.set(customToken);
         }

         if (!StringUtils.isNull(innerToken)) {
            try {
               internal_token.set(innerToken);
               this.setLoginInfo(innerToken);
               return true;
            } catch (Exception var12) {
               Exception e = var12;
               logger.info(e.getMessage());
               throw new Exception("校验token报错，请联系管理员！");
            }
         } else {
            response.sendError(400, "没有传入token!");
            return false;
         }
      }
   }

   public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {
      tempUser.remove();
      tempEnterpirse.remove();
      token.remove();
      internal_token.remove();
      linkRequestId.remove();
      ThreadContext.remove("LinkRequestId");
   }

   private void setLoginInfo(String token) throws Exception {
      String encryptStr = AesUtil.decrypt(token);
      JSONObject jsonObject = JSONObject.parseObject(encryptStr);
      String userJson = jsonObject.getString("user");
      CoreUser user = (CoreUser)JSON.parseObject(userJson, CoreUser.class);
      String enterJson = jsonObject.getString("enterprise");
      String loginInfoKey = "login_info_" + (String)SecurityInterceptor.token.get();
      LinkRedisClient jedis = null;

      try {
         jedis = RedisUtil.getJedis();
         String orgTileJson = jedis.hget(loginInfoKey, "orgTile");
         CoreOrganizationTile orgTile = (CoreOrganizationTile)JSON.parseObject(orgTileJson, CoreOrganizationTile.class);
         user.setCoreOrganizationTile(orgTile);
      } finally {
         if (jedis != null) {
            RedisUtil.returnResource(jedis);
         }

      }

      tempUser.set(user);
      CoreEnterprise enterprise = (CoreEnterprise)JSON.parseObject(enterJson, CoreEnterprise.class);
      if (user == null) {
         throw new RuntimeException("从内部token中获取当前登录用户信息失败，token：" + token);
      } else if (enterprise == null) {
         throw new RuntimeException("从redis获取当前公司账套信息失败，token：" + token);
      } else {
         tempEnterpirse.set(enterprise);
      }
   }

   public static void setUserToken(String key, String userToken) {
   }

   private String splitOrginToken(String orginToken) throws Exception {
      if (StringUtils.isBlank(orginToken)) {
         return null;
      } else {
         String[] orginTokenArray = orginToken.split("\\s+");
         return orginTokenArray.length < 2 ? null : orginTokenArray[1];
      }
   }
}
